Evaluator

Struct Evaluator 

pub struct Evaluator<Wire> { /* private fields */ }
Streaming evaluator using a callback to receive ciphertexts as needed.

Evaluates a garbled circuit on the fly, using messages containing ciphertexts and wires. Parallelizable.

Implementations

Source

impl<Wire: WireLabel> Evaluator<Wire>

pub fn new(channel: &mut Channel<'_>) -> Result<Self>

Create a new Evaluator.