WireMod3

Struct WireMod3 

pub struct WireMod3 { /* private fields */ }
Representation of a mod-3 wire.

We represent a mod-3 wire by 64 mod-3 elements. These elements are stored as follows: the least-significant bits of each element are stored in lsb and the most-significant bits of each element are stored in msb. This representation allows for efficient addition and multiplication as described here by the paper “Hardware Implementation of Finite Fields of Characteristic Three.” D. Page, N.P. Smart. CHES 2002. Link: https://link.springer.com/content/pdf/10.1007/3-540-36400-5_38.pdf.