FancyOutput

Trait FancyOutput 

pub trait FancyOutput: Fancy {
    // Required method
    fn output(
        &mut self,
        x: &Self::Item,
        channel: &mut Channel<'_>,
    ) -> Result<Option<u16>>;

    // Provided method
    fn outputs(
        &mut self,
        xs: &[Self::Item],
        channel: &mut Channel<'_>,
    ) -> Result<Option<Vec<u16>>> { ... }
}
Extension trait for Fancy that provides output operations.

Required Methods

fn output( &mut self, x: &Self::Item, channel: &mut Channel<'_>, ) -> Result<Option<u16>>

Output the value associated with wire x.

Some Fancy implementers don’t actually return output, but they need to be involved in the process, so they can return None.

Provided Methods

fn outputs( &mut self, xs: &[Self::Item], channel: &mut Channel<'_>, ) -> Result<Option<Vec<u16>>>

Output the values associated with a slice of wires.

Some Fancy implementers don’t actually return output, but they need to be involved in the process, so they can return None.

Implementors