pub trait Flatten {
type Item;
// Required method
fn flatten(self) -> Vec<Self::Item>;
}Expand description
Trait for flattening the output of a Circuit into a vector of wires.
pub trait Flatten {
type Item;
// Required method
fn flatten(self) -> Vec<Self::Item>;
}Trait for flattening the output of a Circuit into a vector of wires.