pub struct Evaluator<Wire> { /* private fields */ }Expand description
Streaming evaluator using a callback to receive ciphertexts as needed.
Evaluates a garbled circuit on the fly, using messages containing ciphertexts and wires. Parallelizable.
Implementations