Garbler

Struct Garbler 

pub struct Garbler<RNG, Wire> { /* private fields */ }
Streams garbled circuit ciphertexts through a callback.

Implementations

Source

impl<RNG: CryptoRng + RngCore, Wire: WireLabel + DeserializeOwned> Garbler<RNG, Wire>

pub fn load_deltas(&mut self, filename: &str) -> Result<(), Box<dyn Error>>

Load pre-chosen deltas from a file

Source

impl<RNG: CryptoRng + RngCore, Wire: WireLabel> Garbler<RNG, Wire>

pub fn new(rng: RNG, channel: &mut Channel<'_>) -> Result<Self>

Create a new Garbler.

pub fn delta(&mut self, q: u16) -> Wire

Create a delta if it has not been created yet for this modulus, otherwise just return the existing one.

pub fn get_deltas(self) -> HashMap<u16, Wire>

Get the deltas, consuming the Garbler.

This is useful for reusing wires in multiple garbled circuit instances.

pub fn encode_zero(&mut self, modulus: u16) -> Wire

Output a fresh zero wirelabel associated with the provided modulus.

pub fn bin_encode_zero(&mut self, nbits: usize) -> BinaryBundle<Wire>

Output fresh zero wirelabels associated with a BinaryBundle.