pub struct Garbler<RNG, Wire> { /* private fields */ }Expand description
Streams garbled circuit ciphertexts through a callback.
Implementations
Sourceimpl<RNG: CryptoRng + RngCore, Wire: WireLabel> Garbler<RNG, Wire>
impl<RNG: CryptoRng + RngCore, Wire: WireLabel> Garbler<RNG, Wire>
Sourcepub fn delta(&mut self, q: u16) -> Wire
pub fn delta(&mut self, q: u16) -> Wire
Create a delta if it has not been created yet for this modulus, otherwise just return the existing one.
Sourcepub fn get_deltas(self) -> HashMap<u16, Wire>
pub fn get_deltas(self) -> HashMap<u16, Wire>
Get the deltas, consuming the Garbler.
This is useful for reusing wires in multiple garbled circuit instances.
Sourcepub fn encode_zero(&mut self, modulus: u16) -> Wire
pub fn encode_zero(&mut self, modulus: u16) -> Wire
Output a fresh zero wirelabel associated with the provided modulus.
Sourcepub fn bin_encode_zero(&mut self, nbits: usize) -> BinaryBundle<Wire>
pub fn bin_encode_zero(&mut self, nbits: usize) -> BinaryBundle<Wire>
Output fresh zero wirelabels associated with a BinaryBundle.