pub struct Garbler<RNG, Wire> { /* private fields */ }Expand description
Streams garbled circuit ciphertexts through a callback.
Implementations
Sourceimpl<RNG: CryptoRng + RngCore, Wire: WireLabel> Garbler<RNG, Wire>
impl<RNG: CryptoRng + RngCore, Wire: WireLabel> Garbler<RNG, Wire>
Sourcepub fn delta(&mut self, q: u16) -> Wire
pub fn delta(&mut self, q: u16) -> Wire
Create a delta if it has not been created yet for this modulus, otherwise just return the existing one.
Sourcepub fn get_deltas(self) -> HashMap<u16, Wire>
pub fn get_deltas(self) -> HashMap<u16, Wire>
Get the deltas, consuming the Garbler.
This is useful for reusing wires in multiple garbled circuit instances.
Sourcepub fn send_wire(
&mut self,
wire: &Wire,
channel: &mut Channel<'_>,
) -> Result<()>
pub fn send_wire( &mut self, wire: &Wire, channel: &mut Channel<'_>, ) -> Result<()>
Send a wire over the established channel.
Sourcepub fn encode_wire(&mut self, val: u16, modulus: u16) -> (Wire, Wire)
pub fn encode_wire(&mut self, val: u16, modulus: u16) -> (Wire, Wire)
Encode a wire, producing the zero wire as well as the encoded value.
Sourcepub fn encode_many_wires(
&mut self,
vals: &[u16],
moduli: &[u16],
) -> (Vec<Wire>, Vec<Wire>)
pub fn encode_many_wires( &mut self, vals: &[u16], moduli: &[u16], ) -> (Vec<Wire>, Vec<Wire>)
Encode many wires, producing zero wires as well as encoded values.
Panics
Panics if the length of vals and moduli are not equal.
Sourcepub fn crt_encode_wire(
&mut self,
val: u128,
modulus: u128,
) -> (CrtBundle<Wire>, CrtBundle<Wire>)
pub fn crt_encode_wire( &mut self, val: u128, modulus: u128, ) -> (CrtBundle<Wire>, CrtBundle<Wire>)
Encode a CrtBundle, producing zero wires as well as encoded values.
Sourcepub fn bin_encode_wire(
&mut self,
val: u128,
nbits: usize,
) -> (BinaryBundle<Wire>, BinaryBundle<Wire>)
pub fn bin_encode_wire( &mut self, val: u128, nbits: usize, ) -> (BinaryBundle<Wire>, BinaryBundle<Wire>)
Encode a BinaryBundle, producing zero wires as well as encoded values.